/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "interface.h"
#include "eprintf.h"

Args *args;

Args *getArgs(int argc, char *argv[]){
  int c;
  char *optString = "hvpPsd:a:w:";

  args = (Args *)emalloc(sizeof(Args));
  args->d = DEFAULT_D;
  args->a = DEFAULT_A;
  args->s = 0;
  args->w = 0;
  args->p = 0;
  args->P = 0;
  args->h = 0;
  args->v = 0;
  args->e = 0;

  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 'd':                           /* delimiter */
      args->d = optarg[0];
      break;
    case 'w':                           /* width of sliding window */
      args->w = atoi(optarg);
      break;
    case 's':                           /* summarize result */
      args->s = 1;
      break;
    case 'a':                           /* alphabet */
      if(optarg[0] != 'p')
	args->a = DEFAULT_A;
      else
	args->a = optarg[0];
      break;
    case 'p':                           /* print alphabet and exit */
      args->p = 1;
      break;
    case 'P':                           /* print alphabet at every step */
      args->P = 1;
      break;
    case '?':                           /* fall-through is intentional */
    case 'h':                           /* print help */
      args->h = 1;
      break;
    case 'v':                           /* print version */
      args->v = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  args->inputFiles = argv + optind;
  args->numInputFiles = argc - optind;
  return args;
}


void printUsage(char *version){
  printf("Usage: %s [options] [inputFiles]\n",progname());
  printf("Purpose: Illustrate 'move to front' procedure\n");
  printf("Example: mtf foo.fasta\n");
  printf("Options:\n");
  printf("\t[-d <CHAR> delimiter; default: no delimiter]\n");
  printf("\t[-w <NUM> width of sliding window; default: raw output]\n");
  printf("\t[-s summarize result; default: print raw output]\n");
  printf("\t[-a p|d alphabet, printing (p) or DNA (d); default: %c]\n",DEFAULT_A);
  printf("\t[-p print alphabet and exit]\n");
  printf("\t[-P print alphabet after every coding step]\n");
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-v print program information and exit]\n");
  exit(0);
}

void printSplash(char *version){
  printf("%s %s\n",progname(),version);
  printf("Written by Bernhard Haubold.\n");
  printf("Distributed under the GNU General Public License.\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}
