/***** list.c *************************************
 * Description: List processing.
 * Reference: Sedgewick, R. (1998). Algorithms in
 *   C. Third Edition, Parts 1-4. Addison-Wesley,
 *   p. 102.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Tue Nov 17 09:01:57 2015
 **************************************************/
#include <stdlib.h>
#include <ctype.h>
#include "eprintf.h"
#include "list.h"


int globalPos;

void printList(Node *node){
  Node *np;
  int i,c, x, n;
  
  c = 0;
  for(np=node;np!=NULL;np=np->next)
    c++;
  /* take care of 10^2 */
  if(c >= 100){
    for(i=0;i<100;i++)
      printf(" ");
    n = 0;
    x = 1;
    for(i=100;i<c;i++){
      printf("%d",x);
      n++;
      if(n == 100){
	n = 0;
	x++;
      }
    }
    printf("\n");
  }
  /* take care of 10^1 */
  if(c >= 10){
    for(i=0;i<10;i++)
      printf(" ");
    n = 0;
    x = 1;
    for(i=10;i<c;i++){
      printf("%d",x);
      n++;
      if(n == 10){
	n = 0;
	x++;
      }
    }
    printf("\n");
  }
  /* take care of 10^0 */
  for(i=0;i<c;i++)
    printf("%d", i % 10);
  printf("\n");
  for(np=node;np!=NULL;np=np->next)
    printf("%c",np->item);
  printf("\n");
}

Node *moveToFront(Node *head, char c){
  Node *lp, *dp;

  if(!head){
    globalPos = -1;
    return head;
  }
  globalPos = 0;
  if(head->item == c)
    return head;
  globalPos = 1;
  for(lp = head; lp->next != NULL; lp = lp->next){
    if(lp->next->item == c){
      dp = lp->next;
      lp->next = lp->next->next;
      dp->next = head;
      return dp;
    }
    globalPos++;
  }
  return NULL;
}

Node *newNode(char c){
  Node *n;

  n = (Node *)emalloc(sizeof(Node));
  n->item = c;
  n->next = NULL;

  return n;
}

Node *getAlphabet(char a){
  int i, j, numChar;
  char *dna = "ACGTacgt";
  Node *head, *tail, *node;

  if(a == 'd'){
    head = newNode(dna[0]);
    tail = head;
    for(i=1;i<8;i++){
      node = newNode(dna[i]);
      tail->next = node;
      tail = tail->next;
    }
    return head;
  }

  numChar = 128;
  i = 0;
  while(!isprint(i))
    i++;
  head = newNode(i++);
  tail = head;
  for(j=i;j<numChar;j++){
    if(isprint(j)){
      node = newNode(j);
      tail->next = node;
      tail = tail->next;
    }
  }
  return head;
}

int getPos(){
  return globalPos;
}
